/*
   Class Name:

      CTrackball.

   Created by:

      Allen Sherrod (Programming Ace of www.UltimateGameProgramming.com).

   Description:

      This class allows us to manipulate 3D objects in the scene.
*/


#include"CTrackball.h"
#define TRACKBALL_SIZE 0.7f


CTrackball::CTrackball()
{

}


CTrackball::~CTrackball()
{

}


CQuaternion CTrackball::CalcTrackball(float x1, float y1, float x2, float y2)
{
   CVector4 axis;
   CVector4 p1, p2, difference;
   float phi;
   float length;
   CQuaternion q;

   // First we take the last moust pos point and make it 3D.
   p1.x = x1;
   p1.y = y1;
   p1.z = ProjectPointToSphere(TRACKBALL_SIZE, x1, y1);
   
   // Same as with the last point with the current point.
   p2.x = x2;
   p2.y = y2;
   p2.z = ProjectPointToSphere(TRACKBALL_SIZE, x2, y2);

   // Calculate the different between the two points.
   difference = p1 - p2;

   // Length of the difference.
   length = difference.GetLength() / (2.0f * TRACKBALL_SIZE);

   // Clamp between -1 and 1.
   if(length > 1.0) length = 1.0;
   if(length < -1.0) length = -1.0;

   // Calculate the rotation amount.
   phi = 2.0f * (float)asin(length);

   // Get the cross product between the two 3D points.
   axis.CrossProduct(p2, p1);

   // Create the new quaternion rotation.
   q.CreateQuatFromAxis(axis, phi);

   return q;
}


float CTrackball::ProjectPointToSphere(float radius, float x, float y)
{
   float length = 0.0f;
   float z = 0.0f;

   // This function is used to calculate a z axis from 2 mouse pos.
   // First we need the lenght of the positions, if it is greater or smaller
   // than the radius will determine how the z is calculated.

   length = (float)sqrt(x * x + y * y);

   if(length < radius * 0.70710678118654752440f)
      {
         // Inside the sphere.
         z = (float)sqrt(radius * radius - length * length);
      }
   else
      {
         // On hyperbola.
         radius = radius / 1.41421356237309504880f;
         z = radius * radius / length;
      }
   return z;
}


// Copyright October 2004
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com